<?php
/**
 * @file
 * Contains \Drupal\simple_pixel_tracking\Form\SimplePixelTrackingSettingsForm.
 */
namespace Drupal\simple_pixel_tracking\Form;


use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Extension\ThemeExtensionList;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\ConfigFormBase;
use Symfony\Component\DependencyInjection\ContainerInterface;


class SimplePixelTrackingSettingsForm extends ConfigFormBase {

  /**
   * Config name.
   */
  const CONFIG_NAME = 'config.simple_pixel_tracking';

  /**
   * An extension discovery instance.
   *
   * @var \Drupal\Core\Extension\ThemeExtensionList
   */
  protected $themeList;

  /**
   * Constructs a \Drupal\system\ConfigFormBase object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param \Drupal\Core\Extension\ThemeExtensionList $theme_list
   *   The theme extension list.
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    ThemeExtensionList $theme_list
  ) {
    parent::__construct($config_factory);

    $this->themeList = $theme_list;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('extension.list.theme')
    );
  }

  /**
   * {@inheritDoc}
   */
  public function getFormId() {
    return 'simple_pixel_tracking_settings_form';
  }

  /**
   * {@inheritDoc}
   */
  protected function getEditableConfigNames() {
    return ['config.simple_pixel_tracking'];
  }

  /**
   * {@inheritDoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config(self::CONFIG_NAME);

    $theme_list = $this->themeList->getList();

    $form['regions'] = [
      '#type' => 'details',
      '#open' => TRUE,
      '#title' => $this->t('Theme regions'),
      '#tree' => TRUE,
    ];

    foreach ($theme_list as $key => $theme) {
      $regions = system_region_list($theme, REGIONS_ALL);

      $form['regions'][$key] = [
        '#type' => 'select',
        '#options' => $regions,
        '#title' => $theme->info['name'],
        '#default_value' => $config->get('regions')[$key],
      ];
    }

    $form['url'] = [
      '#type' => 'url',
      '#title' => $this->t('Pixel URL'),
      '#default_value' => $config->get('url'),
      '#description' => 'A URL that leads to the pixel image source.',
      '#required' => TRUE,
    ];

    $form['pid'] = [
      '#type' => 'textfield',
      '#size' => 20,
      '#title' => $this->t('Tracking portal ID'),
      '#default_value' => $config->get('pid'),
      '#description' => $this->t('An ID that marks analytics data.'),
      '#required' => TRUE,
    ];

    $form['pages_wrapper'] = [
      '#type' => 'container',
      'pages_type' => [
        '#type' => 'radios',
        '#title' => $this->t('Tracking pages'),
        '#default_value' => $config->get('pages_type'),
        '#options' => [
          'exclude' => $this->t('Every page except the listed pages'),
          'include' => $this->t('The listed pages only'),
        ],
        '#required' => TRUE,
      ],
      'pages_patterns' => [
        '#type' => 'textarea',
        '#size' => 7,
        '#default_value' => $config->get('pages_patterns'),
        '#description' => $this->t('Add tracking to specific pages.'),
      ],
    ];

    $form['arguments_wrapper'] = [
      '#type' => 'container',
      '#attributes' => [
        'id' => 'tracking-arguments',
      ],
      'title' => [
        '#type' => 'html_tag',
        '#tag' => 'b',
        '#value' => $this->t('Tracking arguments:'),
      ],
      'arguments' => [
        '#type' => 'table',
        '#title' => $this->t('Tracking arguments'),
        '#attributes' => [
          'id' => 'arguments',
        ],
        '#header' => [
          ['data' => '', 'style' => 'width: 5px;'],
          ['data' => $this->t('Name')],
          ['data' => $this->t('Value')],
          ['data' => $this->t('Slot')],
        ],
        '#tabledrag' => [
          [
            'action' => 'order',
            'relationship' => 'sibling',
            'group' => 'arguments-slot',
          ]
        ],
      ],
    ];

    $items = $config->get('arguments');
    if (
      $form_state->isRebuilding() &&
      !empty($arguments = $form_state->getValue('arguments'))
    ) {
      $items = $arguments;
    }

    $cnt = empty($items) ? 1 : count($items);
    for ($id = 0; $id < $cnt; $id++) {
      $form['arguments_wrapper']['arguments'][$id] = [
        '#attributes' => ['class' => ['draggable']],
        'spacer' => [
          '#markup' => '&nbsp;',
        ],
        'name' => [
          '#type' => 'textfield',
          '#size' => 20,
          '#default_value' => $items[$id]['name'],
          '#states' => [
            'required' => ['input[name="arguments[' . $id . '][value]"' => ['empty' => FALSE]],
          ],
        ],
        'value' => [
          '#type' => 'textfield',
          '#size' => 20,
          '#default_value' => $items[$id]['value'],
          '#states' => [
            'required' => ['input[name="arguments[' . $id . '][name]"' => ['empty' => FALSE]],
          ],
        ],
        'slot' => [
          '#type' => 'weight',
          '#default_value' => $id,
          '#title' => $this->t('Analytics slot'),
          '#attributes' => [
            'class' => ['arguments-slot', 'arguments-slot-' . $id],
          ],
        ],
      ];
    }

    $form['add_more_wrapper'] = [
      '#type' => 'container',
      'add_more' => [
        '#type' => 'submit',
        '#value' => $this->t('Add argument'),
        '#name' => 'tracking_add_parameter_id',
        '#submit' => ['::addParameter'],
        '#ajax' => [
          'callback' => '::addArgumentCallback',
          'wrapper' => 'tracking-arguments',
          'effect' => 'fade',
        ],
      ],
      'tokens' => [
        '#theme' => 'token_tree_link',
        '#token_types' => ['simple-pixel-tracking'],
        '#show_restricted' => TRUE,
        '#global_types' => FALSE,
      ],
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * Submit handler for the "Add argument" button.
   *
   * Increments the max counter and causes a rebuild.
   */
  public function addParameter(array &$form, FormStateInterface $form_state) {
    $arguments = $form_state->getValue('arguments');
    $arguments[] = [];
    $form_state->setValue('arguments', $arguments);
    $form_state->setRebuild();
  }

  /**
   * Callback for both ajax custom parameters buttons.
   *
   * Selects and returns the fieldset with the names in it.
   */
  public function addArgumentCallback(array &$form, FormStateInterface $form_state) {
    return $form['arguments_wrapper'];
  }

  public function submitForm(array &$form, FormStateInterface $form_state) {
    $form_state->cleanValues();
    $data = $form_state->getValues();
    $data['arguments'] = array_filter(
      $data['arguments'],
      fn($item) => !empty($item['name']) && !empty($item['value'])
    );
    usort($data['arguments'], fn($a, $b) => $a['slot'] <=> $b['slot']);
    foreach ($data['arguments'] as &$item) {
      unset($item['slot']);
    }
    $config = $this->config(self::CONFIG_NAME);
    $config->setData($data)->save();
  }
}
