<?php

namespace Drupal\simple_pixel_tracking;


use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Theme\ThemeManagerInterface;
use Drupal\Core\Url;
use Drupal\token\Token;
use Drupal\simple_pixel_tracking\Form\SimplePixelTrackingSettingsForm;

/**
 * Implements a pixel tracking manager.
 */
class SimplePixelTrackingManager {

  const PORTAL_ID_PARAM = 'siteid';

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * @var \Drupal\Core\Utility\Token
   */
  protected $token;

  /**
   * The theme manager.
   *
   * @var \Drupal\Core\Theme\ThemeManagerInterface
   */
  protected $themeManager;

  /**
   * Constructs a DevelDumperPluginManager object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory service.
   * @param \Drupal\token\Token $token
   *   The token manager.
   * @param \Drupal\Core\Theme\ThemeManagerInterface $theme_manager
   *   The theme manager.
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    Token $token,
    ThemeManagerInterface $theme_manager
  ) {
    $this->configFactory = $config_factory;
    $this->token = $token;
    $this->themeManager = $theme_manager;
  }

  public function buildTrackingUrl() {
    if (empty($config = $this->configFactory->get(SimplePixelTrackingSettingsForm::CONFIG_NAME))) {
      return FALSE;
    }

    $base = $config->get('url');

    $args= [];
    foreach ($this->getArguments() as $key => $item) {
      $args[] = "{$key}={$item}";
    }

    $target = $base . (stristr($base, '?') ? '&' : '?') . implode('&', $args);
    return str_replace('#', '%23', $target);
  }

  /**
   * Checks if tracking can be added to the page.
   */
  public function checkTrackingAccess() {
    if (empty($config = $this->configFactory->get(SimplePixelTrackingSettingsForm::CONFIG_NAME))) {
      return FALSE;
    }

    if (empty($pages_patterns = $config->get('pages_patterns'))) {
      return TRUE;
    }

    $trim = ' /';
    $type = $config->get('pages_type');
    $current = Url::fromRoute('<current>');
    $path = trim($current->getInternalPath(), $trim);
    $alias = trim($current->toString(), $trim);
    $match = !empty(array_filter(
      preg_split("/\r\n|\n|\r/", $pages_patterns),
      fn($i) => (fnmatch(trim($i, $trim), $path) || fnmatch(trim($i, $trim), $alias))
    ));

    return (($type == 'exclude') && !$match) || (($type == 'include') && $match);
  }

  /**
   * Gets processed arguments.
   */
  public function getThemeRegion() {
    if (empty($config = $this->configFactory->get(SimplePixelTrackingSettingsForm::CONFIG_NAME))) {
      return NULL;
    }

    $theme = $this->themeManager->getActiveTheme()->getName();

    return $config->get('regions')[$theme] ?? NULL;
  }

  /**
   * Gets processed arguments.
   */
  public function getArguments() {
    if (empty($config = $this->configFactory->get(SimplePixelTrackingSettingsForm::CONFIG_NAME))) {
      return [];
    }

    $portal = $config->get('pid');
    $args = [
      'r' => self::random(),
      self::PORTAL_ID_PARAM => $portal,
    ];

    foreach ($config->get('arguments') as $item) {
      $arg = $this->token->replace($item['value'], [], ['clear' => TRUE]);
      $args[$item['name']] = $arg;
    }

    return $args;
  }

  /**
   * Generate random parameter.
   */
  public static function random() {
    return substr(md5(mt_rand()), 0, 7);
  }
}
